# 
## To run this:  "./ns Dropall.tcl"
## To make the s-file:
## cp dropband1.* ~/paper/red/penalty/figures
## cd ~/paper/red/penalty/figures
## csh drop3.com
#

## Nov, 19, problem:
## data.tr has everything listed below. 
## But where is "tcp_drops"?  That should come from finish_flowstats.
## And the infile from that comes from data.f.
## From data.f:
##   tcp_drops A * B bad? 
##   where A/B gives the drop-fraction, the fraction of arriving packets
##   dropped

set verbose 1

#
# Append:
# INPUT:
# data.tr:
#  packet-size 512
#  total_packets_acked 187
#  stop-time 100.1
#  total_drops 11 total_packets 245
# 
# OUTPUT:
# dropband1.data1:  percent of arriving packets dropped, bandwidth in KBps 
#
proc append { infile outfile } {
	set awkCode {
		{
		if ($1=="packet-size") {size = $2;}
		if ($1=="total_packets_acked") {packets = $2;}
		if ($1=="stop-time") {time = $2;}
##		was "tcp_drops" not "total_drops"
		if ($1=="total_drops") { 
			dropFraction = $2/$4;
			bandwidth = packets*size/time;
			print 100*dropFraction, bandwidth/1000
			if ($5=="bad") print "bad";
		}
		}
	} 
	exec awk $awkCode $infile >> $outfile
}

#
## "run" runs an individual simulation.
## "infile" is data.tr
## "outfile" is dropband1.data*
#
proc run { infile outfile tcpType interval packetsize } {
	global verbose
	if {$verbose} {
		puts "./ns Dropband.tcl oneflow $interval $tcpType $packetsize"
	}
	exec ./ns Dropband.tcl oneflow $interval $tcpType $packetsize
	append $infile $outfile
	
}

# 
## "runset" runs a simulation set.
#
proc runset { outfile tcpType packetsize } {
	set infile data.tr
	## Only execute following line to reinitialize data file.
#	exec rm -f $outfile
	## for interval=0.00008, CBR arrival rate is 10 Mbps
	## for interval=0.00019, CBR arrival rate is 4.2 Mbps.
	for {set i 2} {$i <= 9} {incr i 1} {
		run $infile $outfile $tcpType 0.0000$i $packetsize
	}
	for {set i 0} {$i <= 9} {incr i 1} {
		run $infile $outfile $tcpType 0.0001$i $packetsize
        }
	run $infile $outfile $tcpType 0.000085 $packetsize
	run $infile $outfile $tcpType 0.000095 $packetsize
	run $infile $outfile $tcpType 0.000075 $packetsize
	run $infile $outfile $tcpType 0.000065 $packetsize
	run $infile $outfile $tcpType 0.000055 $packetsize
	for {set i 1} {$i <= 9} {incr i 1} {
		run $infile $outfile $tcpType 0.00$i $packetsize
	}
}

#
## "finish" prints a graph of the results of the six simulation sets.
#
proc finish {} {
	set top 1250
	set awkCode { 
	 	{ printf "%8.5f %7.3f\n", $1, $2*(100)/(100-$1) } 
	}

	set rtt 0.06
	set packetsize 1460
	set awkCode1 {
		{
		if (NR==1){
		  for (i=0.0003; i<0.001; i+=0.0001){
		    print i*100, 1.22 * packetsize/(rtt*1000*sqrt(i))
		  }
		  for (i=0.001; i<0.01; i+=0.001){
		    print i*100, 1.22 * packetsize/(rtt*1000*sqrt(i))
		  }
		  for (i=0.01; i<0.10; i+=0.01){
		    print i*100, 1.22 * packetsize/(rtt*1000*sqrt(i))
		  }
	 	}}
	}
	set anyfile dropband1.data1
	exec awk $awkCode1 packetsize=$packetsize rtt=$rtt $anyfile > chart

	set f [open temp.rands w]
	puts $f "TitleText: "
	puts $f "Device: Postscript"
	puts $f \"analysis; flush $f
	exec cat chart >@ $f
	flush $f
	puts $f \n\"4; flush $f
	exec awk $awkCode dropband1.data1 | sort >@ $f ; flush $f
	puts $f \n\"1; flush $f
 	exec awk $awkCode dropband1.data2 | sort >@ $f; flush $f
 	puts $f \n\"2; flush $f
 	exec awk $awkCode dropband1.data3 | sort >@ $f; flush $f
 	puts $f \n\"3; flush $f
# 	close $f
	puts "Calling xgraph ..."
	exec xgraph -ly 0,$top -bb -tk -m -x drop_rate -y arrival_rate \
	  temp.rands &
}

global ns_tcp
set tcpTick 0.01
Agent/TCP set tcpTick_ $tcpTick
##set ns_tcp(tcp-tick) $tcpTick
runset dropband1.data1 sack 1460
## 1: dropband1.data1, SACK TCP, 1460-byte packets
runset dropband1.data2 tahoe 1460
## 2: dropband1.data2, Tahoe, 1460-byte packets
runset dropband1.data3 reno 1460  
## 3: dropband1.data3, Reno, 1460-byte packets
finish

