# 
## To run this:  "./ns Dropallold.tcl"
## To make the s-file:
## cp Dropband.* ~/paper/red/penalty/figures
## cd ~/paper/red/penalty/figures
## csh Drop2.com
#

set verbose 1

#
# Append:
# INPUT:
# data.tr:
#  packet-size 512
#  total_packets_acked 187
#  stop-time 100.1
#  total_drops 11 total_packets 245
# 
# OUTPUT:
# Dropband.data1:  percent of arriving packets dropped, bandwidth in KBps 
#
proc append { infile outfile } {
	set awkCode {
		{
		if ($1=="packet-size") {size = $2;}
		if ($1=="total_packets_acked") {packets = $2;}
		if ($1=="stop-time") {time = $2;}
##		was "tcp_drops" not "total_drops"
		if ($1=="total_drops") { 
			dropFraction = $2/$4;
			bandwidth = packets*size/time;
			print 100*dropFraction, bandwidth/1000
			if ($5=="bad") print "bad";
		}
		}
	} 
	exec awk $awkCode $infile >> $outfile
}

#
## "run" runs an individual simulation.
## "infile" is data.tr
## "outfile" is Dropband.data*
#
proc run { infile outfile tcpType interval packetsize } {
	global verbose
	if {$verbose} {
		puts "./ns Dropband.tcl oneflow $interval $tcpType $packetsize"
	}
	exec ./ns Dropband.tcl oneflow $interval $tcpType $packetsize
	append $infile $outfile
	
}

# 
## "runset" runs a simulation set.
#
proc runset { outfile tcpType packetsize } {
	set infile data.tr
	## Only execute following line to reinitialize data file.
#	exec rm -f $outfile
	## for interval=0.00008, CBR arrival rate is 10 Mbps
	## for interval=0.00019, CBR arrival rate is 4.2 Mbps.
	for {set i 8} {$i <= 9} {incr i 1} {
		run $infile $outfile $tcpType 0.0000$i $packetsize
	}
	for {set i 0} {$i <= 9} {incr i 1} {
		run $infile $outfile $tcpType 0.0001$i $packetsize
        }
	run $infile $outfile $tcpType 0.000085 $packetsize
	run $infile $outfile $tcpType 0.000095 $packetsize
	for {set i 1} {$i <= 2} {incr i 1} {
		run $infile $outfile $tcpType 0.00$i $packetsize
	}
}

#
## "finish" prints a graph of the results of the six simulation sets.
#
proc finish {} {
	set top 1250
	set awkCode { 
	 	{ printf "%8.5f %7.3f\n", $1, $2*(100)/(100-$1) } 
	}

	set rtt 0.06
	set packetsize 1460
	set awkCode1 {
		{
		if (NR==1){
		  for (i=0.0003; i<0.001; i+=0.0001){
		    print i*100, 1.22 * packetsize/(rtt*1000*sqrt(i))
		  }
		  for (i=0.001; i<0.01; i+=0.001){
		    print i*100, 1.22 * packetsize/(rtt*1000*sqrt(i))
		  }
		  for (i=0.01; i<0.10; i+=0.01){
		    print i*100, 1.22 * packetsize/(rtt*1000*sqrt(i))
		  }
	 	}}
	}
	set anyfile Dropband.data1
	exec awk $awkCode1 packetsize=$packetsize rtt=$rtt $anyfile > chart

	set f [open temp.rands w]
	puts $f "TitleText: "
	puts $f "Device: Postscript"
	puts $f \"analysis; flush $f
	exec cat chart >@ $f
	flush $f
	puts $f \n\"4; flush $f
	exec awk $awkCode Dropband.data1 | sort >@ $f ; flush $f
	puts $f \n\"1; flush $f
 	exec awk $awkCode Dropband.data2 | sort >@ $f; flush $f
 	puts $f \n\"2; flush $f
 	exec awk $awkCode Dropband.data3 | sort >@ $f; flush $f
 	puts $f \n\"3; flush $f
        exec awk $awkCode Dropband.data4 | sort >@ $f ; flush $f 
        puts $f \n\"1; flush $f
        exec awk $awkCode Dropband.data5 | sort >@ $f; flush $f 
        puts $f \n\"2; flush $f
        exec awk $awkCode Dropband.data6 | sort >@ $f; flush $f 
        puts $f \n\"3; flush $f
# 	close $f
	puts "Calling xgraph ..."
	exec xgraph -ly 0,$top -bb -tk -m -x drop_rate -y arrival_rate \
	  temp.rands &
}

global ns_tcp
set tcpTick 0.01
Agent/TCP set tcpTick_ $tcpTick
runset Dropband.data1 sack 512
## 1: Dropband.data, SACK TCP, 512-byte packets
runset Dropband.data2 sack 1460
## 2: Dropband.data2, SACK TCP, 1460-byte packets.
runset Dropband.data3 tahoe 512
## 3: Dropband.data3, Tahoe, 512-byte packets
runset Dropband.data4 sack-da 512
## 4: Dropband.data4, SACK TCP, 512-byte packets, delayed-ack sink.
runset Dropband.data5 tahoe 1460
## 5: Dropband.data5, Tahoe, 1460-byte packets.
runset Dropband.data6 sack-da 1460
## 6: Dropband.data6, SACK TCP, delayed-ack sink, 1460-byte packets.
finish

