#
source Setred.v2.tcl
set datafile data.tr

proc create_testnet { queuetype } {
        global ns s1 r1 s4
        set s1 [$ns node]
        set r1 [$ns node]
        set s4 [$ns node]

	$ns duplex-link $s1 $r1 100Mb 1ms DropTail
        set L [$ns duplex-link $r1 $s4 10Mb 28ms $queuetype ]
        [[$ns link $r1 $s4] queue] set limit_ 100
        [[$ns link $s4 $r1] queue] set limit_ 100
}

proc printpkts { tcp file } {
        puts $file "total_packets_acked [$tcp set ack_]"
} 

proc printdrops { fid fmon file } {
        set fcl [$fmon classifier]; # flow classifier
	set flow [$fcl lookup auto 0 0 $fid]
        puts $file \
	"total_drops [$flow set pdrops_] total_packets [$flow set pdepartures_]"
}

#proc printdrops { link file } {
#        puts $file \
#	"total_drops [$link stat 0 drops] total_packets [$link stat 0 packets]" 
#} 

proc printstop { stoptime file } {
        puts $file "stop-time $stoptime"
}

#
# "new_tcp1" creates the TCP connection.
# tcpType "sack" for SACK TCP, 
# tcpType "sack-da" for SACK TCP with delayed-ack sink,
# tcpType "tahoe" for Tahoe TCP, 
# tcpType "tahoe-da" for Tahoe TCP with delayed-ack sink.
#
proc new_tcp1 { startTime source dest window class dump size file stoptime 
  tcpType } {
	global ns
	if { $tcpType == "sack" } {
		set sourceType TCP/Sack1
		set destType TCPSink/Sack1
	} elseif { $tcpType == "sack-da" } {
		set sourceType TCP/Sack1
		set destType TCPSink/Sack1/DelAck
	} elseif { $tcpType == "tahoe" } {
		set sourceType TCP
		set destType TCPSink
	} elseif { $tcpType == "tahoe-da" } {
		set sourceType TCP
		set destType TCPSink/DelAck
	} elseif { $tcpType == "reno" } {
                set sourceType TCP/Reno
                set destType TCPSink
        }
        set tcp [$ns create-connection $sourceType $source $destType $dest $class ]
        $tcp set window_ $window
#	$tcp set tcp-tick $tcpTick
        if {$size > 0}  {$tcp set packetSize_ $size }
        set ftp [$tcp attach-source FTP]
        $ns at $startTime "$ftp start"
#	$ns at $stoptime "printpkts $tcp $file"
        if {$dump == 1 } {puts $file "packet-size [$tcp set packetSize_]"}
	return $tcp
}
proc new_cbr { startTime source dest pktSize interval class } {
    global ns 
    set cbr [$ns create-connection CBR $source LossMonitor $dest $class]
    if {$pktSize > 0} {
        $cbr set packetSize_ $pktSize
    }   
    $cbr set interval_ $interval
    $ns at $startTime "$cbr start"
}

#
# "create_flowstats" sets up the flow manager.
#
# stats: total good packets received by sources.
# knob: arrival rate of CBR flow
proc create_flowstats { stoptime node1 node2 } {
    global ns r1 r2 r1fm flowfile 
    
    set flowfile data.f
    set r1fm [$ns makeflowmon Fid]
    set flowdesc [open $flowfile w]
    $r1fm attach $flowdesc
    $ns attach-fmon [$ns link $node1 $node2] $r1fm; 
    # don't use edrop stats
    $ns at $stoptime "$r1fm dump; close $flowdesc"
}

# proc create_flowstats { stoptime node1 node2 } {
#         global r1 s4 r1fm flowfile
# 
# 	set flowfile data.f
#         set r1fm [new flowmgr class]
#         set flowdesc [open $flowfile w]
#         $r1fm attach $flowdesc
# 	$r1fm ndropcats 2
#         [$ns link $node1 $node2] flow-mgr $r1fm
#         $ns at $stoptime "$r1fm dump"
# }

#
# "finish_flowstats" processes the data from a single simulation.
# infile is data.f
# time fid c=forced/unforced type class src dest pktA byteA CpktD CbyteD
#    TpktA TbyteA TCpktD TCbyteD TpktD TbyteD
# A:arrivals D:drops C:category(forced/unforced) T:totals
# outfile is data.tr
#
proc finish_flowstats { infile outfile } {
	set awkCode {
		{ 
		  if ($3 == 0 && $5 == 0) { 
		      bad_drops = $10; 
		      tcp_packets = $8; 
		      drops = $18;
		      printf "tcp_drops %d tcp_packets %d", drops, tcp_packets;
		  }
		}
	}
	exec awk $awkCode $infile >@ $outfile
}

#
# "test_oneflow" runs the simulation.
# "f" is data.tr
# "flowfile" is data.f
#
proc test_oneflow { interval tcpType packetsize } {
	global ns s1 r1 s4 datafile r1fm flowfile 
	set stoptime 100.1 
	set printtime 100.0
	set testname test_two
	set queuetype RED 
	set queuesize 100
	set seed 0

	set ns [new Simulator]
	
	create_testnet $queuetype 
	create_flowstats $stoptime $r1 $s4
	if {$queuetype == "red"} {
	  set_Red $r1 $s4
	}
	set f [open $datafile w]

	set tcp [new_tcp1 0.0 $s1 $s4 100 0 1 $packetsize $f $stoptime $tcpType]
	new_cbr 1.4 $s1 $s4 100 $interval 1 
	$ns at $stoptime "$r1fm dump; printstop $stoptime $f; \
		printpkts $tcp $f; printdrops 0 $r1fm $f; \
		finish_flowstats $flowfile $f; close $f; exit 0"
##	$ns at $stoptime "$r1fm flush"
##	$ns at $stoptime "finish_flowstats $flowfile $f"
##	$ns at $stoptime "close $f"
##	$ns at $stoptime "exit 0"

	ns-random $seed
	$ns run
}

set numArgs 4
if { $argc < $numArgs || $argc > $numArgs} {
        puts stderr {usage: ns $argv [ arguments ]}
        exit 1
} elseif { $argc == $numArgs } {
	set testname [lindex $argv 0]
	set interval [lindex $argv 1]
	set tcpType [lindex $argv 2]
	set packetsize [lindex $argv 3]
	puts "interval: $interval"
}
if { "[info procs test_$testname]" != "test_$testname" } {
        puts stderr "$testname: no such test: $testname"
}
test_$testname $interval $tcpType $packetsize

